% Copyright (C) 2014  Daniel Filkovic

% This file is part of APAME-WT.

% APAME-WT is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.

% APAME-WT is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with APAME-WT.  If not, see <http://www.gnu.org/licenses/>.

% APAME-WT - Aircraft PAnel MEthod-Wind Turbine
% - 3D Panel Method
% - combination of constant source and constant dipole panels
% - dirichlet's boundary condition
% - spiral vortex trail
% - "far field" substitution of panel with point singularity
%
% Reference:  - Katz, Plotkin; Low Speed Aerodynamics - Second edition
%             - Kesic; Osnove aerodinamike
%             - Jankovic; Mehanika leta zrakoplova
%             - Abbott, Doenhoff; Theory of wing sections

% file apame_wt.m

clear all
close all
clc

% REFERENCE VALUES
span = 1;           % reference span (wing span)
c = 0.8;            % reference length (mean aerodynamic chord)
Sw = 0.8;           % reference surface area (wing surface area)

% FLOW PARAMETERS
V = 10;             % reference speed
ro = 1.225;         % air density
AL = 0*pi/180;      % angle of attack
BETA = 0*pi/180;    % sidewind angle
% velocity components
Vx = V*cos(AL)*cos(BETA);
Vy = V*sin(BETA);
Vz = V*cos(BETA)*sin(AL);
% velocity vertical profile
VA = V;             % constant velocity coefficient
VB = 10;             % linear velocity coefficient
VC = 0;             % quadratic velocity coefficient

% disregard velocity vertical profile if attack or sideslip angles given
if AL ~= 0 || BETA ~= 0
    VA = V;
    VB = 0;
    VC = 0;
end

% SOLVER PARAMETERS
error = 0.000001;   % minimal distance (error)
farfieldfaktor = 5; % "far field" distance in longer panel diagonal

% geometry definition
compIndex = 0;

load wing1

compIndex = compIndex+1;                % geometry component index
M1(compIndex) = 7;                      % number of grid points columnwise (along wingspan)
N1(compIndex) = 25;                     % number of grid points rowwise (along airfoil)
x(:, :, compIndex) = x1;                % x coordinates of grid points
y(:, :, compIndex) = y1;                % y coordinates of grid points
z(:, :, compIndex) = z1;                % z coordinates of grid points
lift(compIndex) = 1;                    % flag to indicate lifting component (1 = lifting)
omega(compIndex) = 1;                   % angular velocity (0 for non-rotating component)
rotAxis(compIndex, :) = [-1, 0, 0];     % rotation axis direction (for rotating body only) (only -x supported)
rotCenter(compIndex, :) = [0, 0, 0];    % rotation axis origin (for rotating body only)
numWake(compIndex) = 10;                % number of released wake panels
wakeLength(compIndex) = 10;             % length of wake (for non-rotational body only)
divWake(compIndex) = 10;                % number of wake panels per rotation cycle (for rotating body only)

compIndex = compIndex+1;                % geometry component index
M1(compIndex) = 7;                      % number of grid points columnwise (along wingspan)
N1(compIndex) = 25;                     % number of grid points rowwise (along airfoil)
x(:, :, compIndex) = x1;                % x coordinates of grid points
y(:, :, compIndex) = y1+10;             % y coordinates of grid points
z(:, :, compIndex) = z1;                % z coordinates of grid points
lift(compIndex) = 1;                    % flag to indicate lifting component (1 = lifting)
omega(compIndex) = 1;                   % angular velocity (0 for non-rotating component)
rotAxis(compIndex, :) = [-1, 0, 0];     % rotation axis direction (for rotating body only) (only -x supported)
rotCenter(compIndex, :) = [0, 0, 0];    % rotation axis origin (for rotating body only)
numWake(compIndex) = 10;                % number of released wake panels
wakeLength(compIndex) = 10;             % length of wake (for non-rotational body only)
divWake(compIndex) = 10;                % number of wake panels per rotation cycle (for rotating body only)

numComp = compIndex;

% analysis is performed for multiple angle positions
% store old body dimensions (avoid wake growth) and geometry (avoid rotation)
N1_old = N1;
x_old = x;
y_old = y;
z_old = z;

angleStep = (0:15:360)*pi/180;
CX_history = zeros(1,length(angleStep));
CY_history = zeros(1,length(angleStep));
CZ_history = zeros(1,length(angleStep));
CL_history = zeros(1,length(angleStep));
CM_history = zeros(1,length(angleStep));
CN_history = zeros(1,length(angleStep));
for angleStepIndex=1:length(angleStep)
    % restore old geometry
    N1 = N1_old;
    x = x_old;
    y = y_old;
    z = z_old;
    
    % rotate body 1
    compIndex = 1;
    for rowIndex=1:N1(compIndex)
        for colIndex=1:M1(compIndex)
            radius = sqrt( y(rowIndex, colIndex, compIndex)^2 + ...
                           z(rowIndex, colIndex, compIndex)^2 );
            phi = atan2( z(rowIndex, colIndex, compIndex) , ...
                         y(rowIndex, colIndex, compIndex) )-angleStep(angleStepIndex);
            y(rowIndex, colIndex, compIndex) = radius*cos(phi);
            z(rowIndex, colIndex, compIndex) = radius*sin(phi);
        end
    end

    % rotate body 2
    compIndex = 2;
    for rowIndex=1:N1(compIndex)
        for colIndex=1:M1(compIndex)
            radius = sqrt( y(rowIndex, colIndex, compIndex)^2 + ...
                           z(rowIndex, colIndex, compIndex)^2 );
            phi = atan2( z(rowIndex, colIndex, compIndex) , ...
                         y(rowIndex, colIndex, compIndex) )-angleStep(angleStepIndex);
            y(rowIndex, colIndex, compIndex) = radius*cos(phi);
            z(rowIndex, colIndex, compIndex) = radius*sin(phi);
        end
    end

    % GEOMETRY CALCULATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    disp('Processing geometry...')
    tic
    % calculation of grid information
    [x, y, z, NBody, N1Body, N1, N, M, numPan, ...
     maxN, maxM, maxN1, maxM1, maxNBody, ...
     FF, S, ...
     u1, u2, u3, ...
     o1, o2, o3, ...
     n1, n2, n3, ...
     p1, p2, p3, ...
     cx, cy, cz, ...
     x1, x2, x3, x4, ...
     y1, y2, y3, y4, ...
     d1, d2, d3, d4] = grid_calculation(x, y, z, M1, N1, ...
                                        lift, omega, rotCenter, ...
                                        V, Vx, Vy, Vz, ...
                                        numWake, wakeLength, divWake, farfieldfaktor);
    disp('Geometry processing time=')
    disp(toc)

    % INFLUENCE COEFFICIENTS CALCULATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    disp('Calculating influence coefficients...')
    tic
    [a, b, sigma] = influence_calculation(numComp, ...
                                          omega, rotCenter, rotAxis, ...
                                          Vy, Vz, VA, VB, VC, ...
                                          NBody, N, M, numPan, ...
                                          FF, S, error, ...
                                          u1, u2, u3, ...
                                          o1, o2, o3, ...
                                          n1, n2, n3, ...
                                          cx, cy, cz, ...
                                          x1, x2, x3, x4, ...
                                          y1, y2, y3, y4, ...
                                          d1, d2, d3, d4);
    disp('Influence coefficients calculation time=')
    disp(toc)

    % SOLVING SYSTEM OF EQUATIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    disp('Solving system of equations...')
    tic
    % assembling right hand side
    RHS = b*sigma;
    clear b sigma
    % solve system of equations
    gamaGlobal = a\RHS;
    clear a RHS FF d1 d2 d3 d4 x1 x2 x3 x4 y1 y2 y3 y4
    % returning dipole values to original matrix
    gama = zeros(maxN, maxM, numComp);
    K = 1;
    for compIndex = 1:numComp
        for rowIndex = 1:NBody(compIndex)
            for colIndex = 1:M(compIndex)
                gama(rowIndex, colIndex, compIndex) = gamaGlobal(K);
                K = K+1;
            end
        end
    end
    clear gamaGlobal
    disp('System of equations solving time=')
    disp(toc)

    % CALCULATION OF SPEEDS, PRESSURES AND FORCES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    tic
    disp('Calculating speeds, pressures and forces...')
    [v, cp, ...
     vx, vy, vz, ...
     CX, CY, CZ, ...
     CL, CM, CN, ...
     CX_comp, CY_comp, CZ_comp, ...
     CL_comp, CM_comp, CN_comp] = post_calculation(gama, ...
                                                   c, span, Sw, V, Vy, Vz, VA, VB, VC, ro, ...
                                                   omega, rotCenter, rotAxis, ...
                                                   S, ...
                                                   numComp, M, NBody, maxM, maxNBody, ...
                                                   u1,u2,u3, ...
                                                   n1,n2,n3, ...
                                                   p1,p2,p3, ...
                                                   o1,o2,o3, ...
                                                   cx,cy,cz);
    disp('Calculation of speeds, pressures and forces time=')
    disp(toc)

    CX_history(angleStepIndex) = CX;
    CY_history(angleStepIndex) = CY;
    CZ_history(angleStepIndex) = CZ;
    CL_history(angleStepIndex) = CL;
    CM_history(angleStepIndex) = CM;
    CN_history(angleStepIndex) = CN;
end


% VALUES DISPLAY %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% pressure coefficient
figure(1)
hold on
axis equal
for compIndex = 1:numComp
    surf(x (1:N1Body(compIndex), 1:M1(compIndex), compIndex), ...
         y (1:N1Body(compIndex), 1:M1(compIndex), compIndex), ...
         z (1:N1Body(compIndex), 1:M1(compIndex), compIndex), ...
         cp(1:NBody(compIndex),  1:M(compIndex),  compIndex))
    %caxis([-0.6, 0.9])
    caxis([min(min(cp(1:NBody(1), 1:M(1), 1))) max(max(cp(1:NBody(1), 1:M(1), 1)))])
end
% velocity magnitude
figure(2)
hold on
axis equal
for compIndex = 1:numComp
    surf(x(1:N1Body(compIndex), 1:M1(compIndex), compIndex), ...
         y(1:N1Body(compIndex), 1:M1(compIndex), compIndex), ...
         z(1:N1Body(compIndex), 1:M1(compIndex), compIndex), ...
         v(1:NBody(compIndex),  1:M(compIndex),  compIndex))
    %caxis([10, 45])
    caxis([min(min(v(1:NBody(1),1:M(1),1))) max(max(v(1:NBody(1),1:M(1),1)))])
end
% grid display
figure(3)
hold on
axis equal
for compIndex = 1:numComp
    mesh(x(1:N1(compIndex), 1:M1(compIndex), compIndex), ...
         y(1:N1(compIndex), 1:M1(compIndex), compIndex), ...
         z(1:N1(compIndex), 1:M1(compIndex), compIndex), ...
         x(1:N1(compIndex), 1:M1(compIndex), compIndex));
end
