% Copyright (C) 2014  Daniel Filkovic

% This file is part of APAME-WT.

% APAME-WT is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.

% APAME-WT is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with APAME-WT.  If not, see <http://www.gnu.org/licenses/>.

% file influence_calculation.m

function [a, b, sigma] = influence_calculation(numComp, ...
                                               omega, rotCenter, rotAxis, ...
                                               Vy, Vz, VA, VB, VC, ...
                                               NBody, N, M, numPan, ...
                                               FF, S, error, ...
                                               u1, u2, u3, ...
                                               o1, o2, o3, ...
                                               n1, n2, n3, ...
                                               cx, cy, cz, ...
                                               x1, x2, x3, x4, ...
                                               y1, y2, y3, y4, ...
                                               d1, d2, d3, d4)

% allocate doublet and source influence coefficients
a = zeros(numPan, numPan, 'single');
b = zeros(numPan, numPan, 'single');

% calculate influence coefficients
%   A suffix denoted influencing component/panel
%   B                influenced
A = 0;
for compIndexA = 1:numComp
    for rowIndexA = 1:NBody(compIndexA)
        for colIndexA = 1:M(compIndexA)
            A = A+1;
            B = 0;
            for compIndexB = 1:numComp
                for rowIndexB = 1:NBody(compIndexB)
                    for colIndexB = 1:M(compIndexB)
                        B = B+1;
                        % calculate distance of panel A to panel B
                        cxAcxB = cx(rowIndexB, colIndexB, compIndexB) - ...
                                 cx(rowIndexA, colIndexA, compIndexA) ;
                        cyAcyB = cy(rowIndexB, colIndexB, compIndexB) - ...
                                 cy(rowIndexA, colIndexA, compIndexA) ;
                        czAczB = cz(rowIndexB, colIndexB, compIndexB) - ...
                                 cz(rowIndexA, colIndexA, compIndexA) ;
                        
                        % convert this distance to panel coordinate system
                        cpx = cxAcxB*u1(rowIndexA, colIndexA, compIndexA) + ...
                              cyAcyB*u2(rowIndexA, colIndexA, compIndexA) + ...
                              czAczB*u3(rowIndexA, colIndexA, compIndexA) ;
                        cpy = cxAcxB*o1(rowIndexA, colIndexA, compIndexA) + ...
                              cyAcyB*o2(rowIndexA, colIndexA, compIndexA) + ...
                              czAczB*o3(rowIndexA, colIndexA, compIndexA) ;
                        cpz = cxAcxB*n1(rowIndexA, colIndexA, compIndexA) + ...
                              cyAcyB*n2(rowIndexA, colIndexA, compIndexA) + ...
                              czAczB*n3(rowIndexA, colIndexA, compIndexA) ;
                        
                        % check "far field" condition with this distance
                        rad = cpx^2+cpy^2+cpz^2;
                        radSqrt = realsqrt(rad);
                        if radSqrt > FF(rowIndexA, colIndexA, compIndexA)
                            % "far field" condition met
                            Smult = 0.079577471545948*S(rowIndexA, colIndexA, compIndexA);
                            a(B, A) = Smult*cpz*rad^(-1.5);
                            b(B, A) = Smult/radSqrt;
                        else
                            % calculate panel equations
                            cpx1 = cpx - x1(rowIndexA, colIndexA, compIndexA);
                            cpx2 = cpx - x2(rowIndexA, colIndexA, compIndexA);
                            cpx3 = cpx - x3(rowIndexA, colIndexA, compIndexA);
                            cpx4 = cpx - x4(rowIndexA, colIndexA, compIndexA);
                            cpy1 = cpy - y1(rowIndexA, colIndexA, compIndexA);
                            cpy2 = cpy - y2(rowIndexA, colIndexA, compIndexA);
                            cpy3 = cpy - y3(rowIndexA, colIndexA, compIndexA);
                            cpy4 = cpy - y4(rowIndexA, colIndexA, compIndexA);
                            e1 = cpx1^2+cpz^2;
                            e2 = cpx2^2+cpz^2;
                            e3 = cpx3^2+cpz^2;
                            e4 = cpx4^2+cpz^2;
                            r1 = realsqrt(e1 + cpy1^2);
                            r2 = realsqrt(e2 + cpy2^2);
                            r3 = realsqrt(e3 + cpy3^2);
                            r4 = realsqrt(e4 + cpy4^2);
                            h1 = cpx1*cpy1;
                            h2 = cpx2*cpy2;
                            h3 = cpx3*cpy3;
                            h4 = cpx4*cpy4;
                            x21 = x2(rowIndexA, colIndexA, compIndexA) - x1(rowIndexA, colIndexA, compIndexA);
                            x32 = x3(rowIndexA, colIndexA, compIndexA) - x2(rowIndexA, colIndexA, compIndexA);
                            x43 = x4(rowIndexA, colIndexA, compIndexA) - x3(rowIndexA, colIndexA, compIndexA);
                            x14 = x1(rowIndexA, colIndexA, compIndexA) - x4(rowIndexA, colIndexA, compIndexA);
                            y21 = y2(rowIndexA, colIndexA, compIndexA) - y1(rowIndexA, colIndexA, compIndexA);
                            y32 = y3(rowIndexA, colIndexA, compIndexA) - y2(rowIndexA, colIndexA, compIndexA);
                            y43 = y4(rowIndexA, colIndexA, compIndexA) - y3(rowIndexA, colIndexA, compIndexA);
                            y14 = y1(rowIndexA, colIndexA, compIndexA) - y4(rowIndexA, colIndexA, compIndexA);
                            if d1(rowIndexA, colIndexA, compIndexA) < error
                                a1 = 0;
                                b1 = 0;
                            else
                                F = y21*e1 - x21*h1;
                                G = y21*e2 - x21*h2;
                                a1 = atan2(cpz*x21*(F*r2-G*r1), cpz^2*x21^2*r1*r2+F*G);
                                b1 = (cpx1*y21-cpy1*x21)/d1(rowIndexA, colIndexA, compIndexA)*reallog((r1+r2+d1(rowIndexA, colIndexA, compIndexA))/(r1+r2-d1(rowIndexA, colIndexA, compIndexA)));
                            end
                            if d2(rowIndexA, colIndexA, compIndexA) < error
                                a2 = 0;
                                b2 = 0;
                            else
                                F = y32*e2 - x32*h2;
                                G = y32*e3 - x32*h3;
                                a2 = atan2(cpz*x32*(F*r3-G*r2), cpz^2*x32^2*r2*r3+F*G);
                                b2 = (cpx2*y32-cpy2*x32)/d2(rowIndexA, colIndexA, compIndexA)*reallog((r2+r3+d2(rowIndexA, colIndexA, compIndexA))/(r2+r3-d2(rowIndexA, colIndexA, compIndexA)));
                            end
                            if d3(rowIndexA, colIndexA, compIndexA) < error
                                a3 = 0;
                                b3 = 0;
                            else
                                F = y43*e3 - x43*h3;
                                G = y43*e4 - x43*h4;
                                a3 = atan2(cpz*x43*(F*r4-G*r3), cpz^2*x43^2*r3*r4+F*G);
                                b3 = (cpx3*y43-cpy3*x43)/d3(rowIndexA, colIndexA, compIndexA)*reallog((r3+r4+d3(rowIndexA, colIndexA, compIndexA))/(r3+r4-d3(rowIndexA, colIndexA, compIndexA)));
                            end
                            if d4(rowIndexA, colIndexA, compIndexA) < error
                                a4 = 0;
                                b4 = 0;
                            else
                                F = y14*e4 - x14*h4;
                                G = y14*e1 - x14*h1;
                                a4 = atan2(cpz*x14*(F*r1-G*r4), cpz^2*x14^2*r4*r1+F*G);
                                b4 = (cpx4*y14-cpy4*x14)/d4(rowIndexA, colIndexA, compIndexA)*reallog((r4+r1+d4(rowIndexA, colIndexA, compIndexA))/(r4+r1-d4(rowIndexA, colIndexA, compIndexA)));
                            end
                            dipol = -(a1+a2+a3+a4)*0.079577471545948;
                            if abs(cpz) < error
                                dipol = 0;
                            end
                            b(B, A) = -(b1+b2+b3+b4)*0.079577471545948-cpz*dipol;
                            
                            % check if panel is influencing on itself
                            if A == B
                                dipol = -0.5;
                            end
                            a(B, A) = dipol;
                        end
                    end
                end
            end
        end
    end
end

% addition of trail wake influence coefficients
for compIndexA = 1:numComp
    for rowIndexA = 1:N(compIndexA)
        for colIndexA = 1:M(compIndexA)
            B = 0;
            for compIndexB = 1:numComp
                for rowIndexB = 1:NBody(compIndexB)
                    for colIndexB = 1:M(compIndexB)
                        B = B+1;
                        if rowIndexA > NBody(compIndexA)
                            % calculate distance of panel A to panel B
                            cxAcxB = cx(rowIndexB, colIndexB, compIndexB) - ...
                                     cx(rowIndexA, colIndexA, compIndexA) ;
                            cyAcyB = cy(rowIndexB, colIndexB, compIndexB) - ...
                                     cy(rowIndexA, colIndexA, compIndexA) ;
                            czAczB = cz(rowIndexB, colIndexB, compIndexB) - ...
                                     cz(rowIndexA, colIndexA, compIndexA) ;

                            % convert this distance to panel coordinate system
                            cpx = cxAcxB*u1(rowIndexA, colIndexA, compIndexA) + ...
                                  cyAcyB*u2(rowIndexA, colIndexA, compIndexA) + ...
                                  czAczB*u3(rowIndexA, colIndexA, compIndexA) ;
                            cpy = cxAcxB*o1(rowIndexA, colIndexA, compIndexA) + ...
                                  cyAcyB*o2(rowIndexA, colIndexA, compIndexA) + ...
                                  czAczB*o3(rowIndexA, colIndexA, compIndexA) ;
                            cpz = cxAcxB*n1(rowIndexA, colIndexA, compIndexA) + ...
                                  cyAcyB*n2(rowIndexA, colIndexA, compIndexA) + ...
                                  czAczB*n3(rowIndexA, colIndexA, compIndexA) ;

                            % check if influenced point lies on panel plane
                            if abs(cpz) < error
                                dipol = 0;
                            else
                                % check "far field" condition with this distance
                                rad = cpx^2+cpy^2+cpz^2;
                                if realsqrt(rad) > FF(rowIndexA, colIndexA, compIndexA)
                                    % "far field" condition met
                                    dipol = 0.079577471545948*S(rowIndexA, colIndexA, compIndexA)*cpz*rad^(-1.5);
                                else
                                    % calculate panel equations
                                    cpx1 = cpx - x1(rowIndexA, colIndexA, compIndexA);
                                    cpx2 = cpx - x2(rowIndexA, colIndexA, compIndexA);
                                    cpx3 = cpx - x3(rowIndexA, colIndexA, compIndexA);
                                    cpx4 = cpx - x4(rowIndexA, colIndexA, compIndexA);
                                    cpy1 = cpy - y1(rowIndexA, colIndexA, compIndexA);
                                    cpy2 = cpy - y2(rowIndexA, colIndexA, compIndexA);
                                    cpy3 = cpy - y3(rowIndexA, colIndexA, compIndexA);
                                    cpy4 = cpy - y4(rowIndexA, colIndexA, compIndexA);
                                    e1 = cpx1^2+cpz^2;
                                    e2 = cpx2^2+cpz^2;
                                    e3 = cpx3^2+cpz^2;
                                    e4 = cpx4^2+cpz^2;
                                    r1 = realsqrt(e1 + cpy1^2);
                                    r2 = realsqrt(e2 + cpy2^2);
                                    r3 = realsqrt(e3 + cpy3^2);
                                    r4 = realsqrt(e4 + cpy4^2);
                                    h1 = cpx1*cpy1;
                                    h2 = cpx2*cpy2;
                                    h3 = cpx3*cpy3;
                                    h4 = cpx4*cpy4;
                                    x21 = x2(rowIndexA, colIndexA, compIndexA) - x1(rowIndexA, colIndexA, compIndexA);
                                    x32 = x3(rowIndexA, colIndexA, compIndexA) - x2(rowIndexA, colIndexA, compIndexA);
                                    x43 = x4(rowIndexA, colIndexA, compIndexA) - x3(rowIndexA, colIndexA, compIndexA);
                                    x14 = x1(rowIndexA, colIndexA, compIndexA) - x4(rowIndexA, colIndexA, compIndexA);
                                    y21 = y2(rowIndexA, colIndexA, compIndexA) - y1(rowIndexA, colIndexA, compIndexA);
                                    y32 = y3(rowIndexA, colIndexA, compIndexA) - y2(rowIndexA, colIndexA, compIndexA);
                                    y43 = y4(rowIndexA, colIndexA, compIndexA) - y3(rowIndexA, colIndexA, compIndexA);
                                    y14 = y1(rowIndexA, colIndexA, compIndexA) - y4(rowIndexA, colIndexA, compIndexA);
                                    if d1(rowIndexA, colIndexA, compIndexA) < error
                                        a1 = 0;
                                    else
                                        F = y21*e1 - x21*h1;
                                        G = y21*e2 - x21*h2;
                                        a1 = atan2(cpz*x21*(F*r2-G*r1), cpz^2*x21^2*r1*r2+F*G);
                                    end
                                    if d2(rowIndexA, colIndexA, compIndexA) < error
                                        a2 = 0;
                                    else
                                        F = y32*e2 - x32*h2;
                                        G = y32*e3 - x32*h3;
                                        a2 = atan2(cpz*x32*(F*r3-G*r2), cpz^2*x32^2*r2*r3+F*G);
                                    end
                                    if d3(rowIndexA, colIndexA, compIndexA) < error
                                        a3 = 0;
                                    else
                                        F = y43*e3 - x43*h3;
                                        G = y43*e4 - x43*h4;
                                        a3 = atan2(cpz*x43*(F*r4-G*r3), cpz^2*x43^2*r3*r4+F*G);
                                    end
                                    if d4(rowIndexA, colIndexA, compIndexA) < error
                                        a4 = 0;
                                    else
                                        F = y14*e4 - x14*h4;
                                        G = y14*e1 - x14*h1;
                                        a4 = atan2(cpz*x14*(F*r1-G*r4), cpz^2*x14^2*r4*r1+F*G);
                                    end
                                    dipol = -(a1+a2+a3+a4)*0.079577471545948;
                                end
                            end
                            
                            % find indexes of trailing upper and lower
                            % panels
                            skip = 0;
                            if compIndexA>1
                                % count panels from previous bodies
                                for i=1:compIndexA-1
                                    skip = skip+NBody(compIndexA-1)*M(compIndexA-1);
                                end
                            end
                            A_lower = skip+colIndexA;
                            A_upper = skip+(NBody(compIndexA)-1)*M(compIndexA)+colIndexA;
                            
                            % subtract dipole value from lower trailing panel
                            a(B, A_lower) = a(B, A_lower) - dipol;
                            % add dipole value to upper trailing panel
                            a(B, A_upper) = a(B, A_upper) + dipol;
                        end
                    end
                end
            end
        end
    end
end

% calculation of source strengths
sigma = zeros(numPan, 1, 'single');
radVec = zeros(3, 1);
A = 1;
for compIndex = 1:numComp
    for rowIndex = 1:NBody(compIndex)
        for colIndex = 1:M(compIndex)
            % calculate local velocity component
            VxLocal = VA+VB*cz(rowIndex, colIndex, compIndex)+VC*cz(rowIndex, colIndex, compIndex)^2;
            % check if rotational component
            if (omega(compIndex) == 0)
                sigma(A) = n1(rowIndex, colIndex, compIndex)*VxLocal + ...
                           n2(rowIndex, colIndex, compIndex)*Vy      + ...
                           n3(rowIndex, colIndex, compIndex)*Vz      ;
            else
                % subtract rotation influence on free stream velocity
                % velocity vector is cross-product of rotational axis vector (of
                % rotational velocity magnitude) and radius vector (vector from
                % rotational axis origin to panel center)
            
                % make sure that rotational axis unit vector is of unit magnitude
                rotAxisMagn = sqrt(rotAxis(compIndex, 1)^2 + ...
                                   rotAxis(compIndex, 2)^2 + ...
                                   rotAxis(compIndex, 3)^2 );
                if (rotAxisMagn ~= 1)
                    rotAxis(compIndex) = rotAxis(compIndex)/rotAxisMagn;
                end

                % multiply it by rotational velocity
                vecOmega = rotAxis(compIndex, :)*omega(compIndex);

                % get radius vector
                radVec(1) = cx(rowIndex, colIndex, compIndex) - rotCenter(compIndex, 1);
                radVec(2) = cy(rowIndex, colIndex, compIndex) - rotCenter(compIndex, 2);
                radVec(3) = cz(rowIndex, colIndex, compIndex) - rotCenter(compIndex, 3);

                % get cross product
                tangVel = cross(vecOmega, radVec);

                % modify free stream velocity
                sigma(A) = n1(rowIndex, colIndex, compIndex)*(VxLocal-tangVel(1)) + ...
                           n2(rowIndex, colIndex, compIndex)*(Vy     -tangVel(2)) + ...
                           n3(rowIndex, colIndex, compIndex)*(Vz     -tangVel(3)) ;
            end
            A = A+1;
        end
    end
end
